"""
    This file is part of Accountant.

    Accountant is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Accountant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with Accountant.  If not, see <http://www.gnu.org/licenses/>.
"""
from flask.ext.restful import Resource, fields, marshal_with_field

from accountant import db

from .. import api

from ..models.accounts import Account
from ..models.operations import Operation

from ..fields import Object

from .models import (account_model, solds_model, balance_model,
                     category_model, ohlc_model)
from .parsers import account_parser, range_parser
from .users import requires_auth


class AccountListResource(Resource):
    @requires_auth
    @marshal_with_field(fields.List(Object(account_model)))
    def get(self):
        """
        Returns accounts with their balances.
        """
        return Account.query().all(), 200

    @requires_auth
    @marshal_with_field(Object(account_model))
    def post(self):
        """
        Create a new account.
        """
        data = account_parser.parse_args()

        account = Account(**data)

        db.session.add(account)

        # Flush session to have id in account.
        db.session.flush()

        # Return account data.
        return account, 201


class AccountResource(Resource):
    @requires_auth
    @marshal_with_field(Object(account_model))
    def get(self, id):
        """
        Get account.
        """
        account = Account.query().get(id)

        if not account:
            return None, 404

        return account, 200

    @requires_auth
    @marshal_with_field(Object(account_model))
    def post(self, id):
        data = account_parser.parse_args()

        assert (id not in data or data.id is None
                or data.id == id)

        # Need to get the object to update it.
        account = Account.query().get(id)

        if not account:
            return None, 404

        # SQLAlchemy objects ignore __dict__.update() with merge.
        for k, v in data.items():
            setattr(account, k, v)

        db.session.merge(account)

        # Return account.
        return account, 200

    @requires_auth
    @marshal_with_field(Object(account_model))
    def delete(self, id):
        # Need to get the object to update it.
        account = Account.query().get(id)

        if not account:
            return None, 404

        db.session.delete(account)

        return None, 204


api.add_resource(AccountListResource, '/account')
api.add_resource(AccountResource, '/account/<int:id>')


class SoldsResource(Resource):
    @requires_auth
    @marshal_with_field(Object(solds_model))
    def get(self, id):
        """
        Get solds for a specific account and date range.
        """
        account = Account.query().get(id)

        if not account:
            return None, 404

        # Note: if we don't pass the code, the result is seen as a tuple and
        # causes error on marshalling.
        return account.solds(), 200


class BalanceResource(Resource):
    @requires_auth
    @marshal_with_field(Object(balance_model))
    def get(self, id):
        """
        Get account balance for a specific date range.
        """
        account = Account.query().get(id)

        if not account:
            return None, 404

        data = range_parser.parse_args()

        # Note: if we don't pass the code, the result is seen as a tuple and
        # causes error on marshalling.
        return account.balance(**data), 200


class CategoryResource(Resource):
    @requires_auth
    @marshal_with_field(fields.List(Object(category_model)))
    def get(self, id):
        data = range_parser.parse_args()

        return Operation.get_categories_for_range(id, **data).all()


class OHLCResource(Resource):
    @requires_auth
    @marshal_with_field(fields.List(Object(ohlc_model)))
    def get(self, id):
        data = range_parser.parse_args()

        return Operation.get_ohlc_per_day_for_range(id, **data).all()


api.add_resource(SoldsResource, "/account/<int:id>/solds")
api.add_resource(BalanceResource, "/account/<int:id>/balance")
api.add_resource(CategoryResource, "/account/<int:id>/category")
api.add_resource(OHLCResource, "/account/<int:id>/ohlc")
